import {
  fetchConsumeList,
  fetchReportList,
  fetchTitleList,
} from '@/api/dashboard/dashboard';
import customerRecharge from './CustomerRecharge/CustomerRechargeStore';
import customerConsumption from './CustomerConsumption/CustomerConsumptionStore';
import customerFeedback from './CustomerFeedback/CustomerFeedbackStore';
import initFiltersStore from '@/store/modules/filters';
import { getFilters, formatDate } from '@/utils/index';

const GET_CONSUME_DATA = 'GET_CONSUME_DATA';
const GET_REPORT_DATA = 'GET_REPORT_DATA';
const GET_TITLE_DATA = 'GET_TITLE_DATA';
const GET_PERCENT_DATA = 'GET_PERCENT_DATA';
const GET_EUIPMENT_DATA = 'GET_EUIPMENT_DATA';
const GET_CAMPUS_DATA = 'GET_CAMPUS_DATA';

const initFilters = () => ({
  operateId: undefined,
  timeType: 0, // 0 本日，1 本月， 2 本年
});
const filtersStore = initFiltersStore(initFilters);
const nowTime = new Date();

const state = () => ({
  consume: {
    areaName: '',
    consumeAfterVos: [],
    consumeHardVos: [],
    list: [],
    percentList: [],
  },
  report: {
    year: new Date().getFullYear(),
    month: new Date().getMonth() + 1,
    registePerDay: [],
    registePerMonth: [],
    activeUser: [],
  },
  title: {
    activeCount: 0,
    countArea: 0,
    countCustomer: 0,
  },
});

const getters = {
  reportYear: state => state.report.year,
  reportMonth: state => state.report.month,
  registePerDay: state => state.report.registePerDay,
  registePerMonth: state => state.report.registePerMonth,
  activeUser: state => state.report.activeUser,
  titleData: state => state.title,
  areaName: state => state.consume.areaName,
  consumeAfterVos: state => state.consume.consumeAfterVos,
  consumeHardVos: state => state.consume.consumeHardVos,
  campusRankList: state => state.consume.list,
  seviceRatioList: state => state.consume.percentList,
};

const actions = {
  fetchConsumeList({ commit, dispatch, getters }, data) {
    if (data) {
      const { timeType, operateId } = data;
      (timeType || timeType == 0) &&
        dispatch('updateFilters', {
          timeType: timeType,
        });
      dispatch('updateFilters', {
        operateId,
      });
    }
    let entity = getFilters(getters.filters);
    switch (entity.timeType) {
      case 1:
        entity.month = formatDate(nowTime, 'yyyy-MM');
        break;
      case 2:
        entity.year = formatDate(nowTime, 'yyyy');
        break;
      case 0:
      default:
        entity.date = formatDate(nowTime, 'yyyy-MM-dd');
        break;
    }
    if (data && data.updatePercentList) {
      entity.areaId = data.areaId;
    }
    if (data && data.updateEuipmentList) {
      entity.serviceId = data.serviceId;
    }
    return fetchConsumeList({
      data: {
        ...entity,
      },
    }).then(res => {
      let list = res.data;
      if (data && data.updatePercentList) {
        commit(GET_PERCENT_DATA, list);
        commit(GET_EUIPMENT_DATA, list);
        return;
      }
      if (data && data.updateEuipmentList) {
        commit(GET_EUIPMENT_DATA, list);
        return;
      }
      commit(GET_CONSUME_DATA, list);
    });
  },
  fetchReportList({ commit }, entity) {
    return fetchReportList({
      params: {
        ...entity,
      },
    }).then(res => {
      let list = res.data;
      commit(GET_REPORT_DATA, list);
    });
  },
  fetchTitleList({ commit }) {
    return fetchTitleList().then(res => {
      let data = res.data;
      commit(GET_TITLE_DATA, data);
    });
  },
  updateCampusName({ commit }, name) {
    commit(GET_CAMPUS_DATA, name);
  },
};

const mutations = {
  [GET_CONSUME_DATA]: (state, data) => {
    state.consume.consumeHardVos = data.consumeHardVos
      ? data.consumeHardVos
      : [];
    state.consume.consumeAfterVos = data.consumeAfterVos
      ? data.consumeAfterVos
      : [];
    let campusList = data.list ? data.list : [];
    if (campusList.length > 10) {
      let arrStart = campusList.slice(0, 10);
      let arrEnd = campusList.slice(10).reduce(
        (item, curentVal) => ({
          ...item,
          count: item.count + curentVal.count,
        }),
        {
          areaName: '其他校区',
          count: 0,
        }
      );
      campusList = [...arrStart, arrEnd];
    }
    state.consume.list = campusList;
    state.consume.percentList = data.percentList ? data.percentList : [];
  },
  [GET_PERCENT_DATA]: (state, data) => {
    state.consume.percentList = data.percentList;
  },
  [GET_EUIPMENT_DATA]: (state, data) => {
    state.consume.consumeHardVos = data.consumeHardVos;
    state.consume.consumeAfterVos = data.consumeAfterVos;
  },
  [GET_REPORT_DATA]: (state, data) => {
    state.report.year = data.year;
    state.report.month = data.month;
    let dayList = new Array(31).fill({ count: 0 });
    dayList = dayList.map((v, idx) => ({ ...v, days: idx + 1 }));
    data.dayVos.map(val => {
      let idx = Number(val.days.substr(-2)) - 1;
      dayList[idx].count = val.count;
    });
    state.report.registePerDay = dayList;
    state.report.registePerMonth = data.list;
    state.report.activeUser = data.listCount;
  },
  [GET_TITLE_DATA](state, data) {
    state.title = data;
  },
  [GET_CAMPUS_DATA](state, name) {
    state.consume.areaName = name;
  },
};

export default {
  namespaced: true,
  modules: {
    filtersStore,
    customerRecharge,
    customerConsumption,
    customerFeedback,
  },
  state,
  getters,
  actions,
  mutations,
};
